@@ -1,4 +1,5 @@
-;;;; 	Copyright (C) 1997 Maciej Stachowiak
+;;;; $Id: decor.scm,v 1.8 1999/10/17 21:42:36 gjb Exp $
+;;;; Copyright (C) 1997-1998 Maciej Stachowiak and Greg J. Badros
 ;;;; 
 ;;;; This program is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -19,11 +20,13 @@
 
 
 (define-module (app scwm decor)
+  :use-module (app scwm style)
   :use-module (app scwm style-options))
 
 
 
 (define-public (call-with-decor decor thunk)
+  "Eval THUNK using DECOR as the current decor."
   (let* ((old-decor (current-decor))
          (swap-in-decor (lambda () 
                           (set-current-decor! decor)))
@@ -34,4 +37,4 @@
 (defmacro-public with-decor (decor . body)
   `(call-with-decor ,decor (lambda () ,@body)))
 
-(add-window-style-option #:use-decor (lambda (d w) (set-window-decor! w d)))
+(add-window-both-option #:use-decor (lambda (d w) (set-window-decor! w d)))
