@@ -1,4 +1,5 @@
-;;;; 	Copyright (C) 1997 Maciej Stachowiak
+;;;; $Id: face.scm,v 1.17 2000/01/13 07:28:06 gjb Exp $
+;;;; Copyright (C) 1997, 1998, 1999, 2000 Maciej Stachowiak and Greg J. Badros
 ;;;; 
 ;;;; This program is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -20,15 +21,40 @@
 
 (define-module (app scwm face)
   :use-module (app scwm base)
+  :use-module (app scwm defoption)
   :use-module (app scwm optargs))
 
 
+(define-scwm-group face "Decorations")
 
+(define-scwm-option *window-font* (make-font "fixed")
+  "The default window titlebar font."
+  #:type 'font
+  #:group 'face
+  #:setter (lambda (font) (set-title-font! font))
+  #:getter (lambda () (title-font)))
 
 (define*-public (title-style #&key font height justify
 			     (active-up '()) 
 			     (active-down '()) 
-			     (inactive '()) . rest)
+			     (inactive '()) #&allow-other-keys #&rest rest)
+  "Set the title style in the current decor.
+FONT is the window title font, a font object or a string.
+HEIGHT is the height of the title bar, in points.
+JUSTIFY is one of 'left, 'right, or 'center.
+This function also takes the keyword arguments #:relief, #:solid,
+#:gradient, #:h-gradient, #:v-gradient, and #:pixmap, with effects
+as described under the `Face Flags' and `Face Specification Flags'
+concepts (except that #:pixmap is always tiled, and #:h-gradient
+and #:v-gradient are horizontal and vertical gradients).  These
+extra keyword arguments can either be included in the main argument
+list, or lists of these arguments can be used as the ACTIVE-UP,
+ACTIVE-DOWN, or INACTIVE arguments.  For instance:
+  (title-style #:solid \"red\" #:inactive (list #:solid \"green\"))
+would set the title to be red in either active state, and green
+in the inactive state, leaving FONT, HEIGHT, and JUSTIFY in their
+current state, and returning everything else about the title
+to their default state."
   (if (bound? font)
       (set-title-font! font))
   (if (bound? height) 
@@ -41,7 +67,14 @@
 
 
 (define*-public (border-style #&key (active '())  
-			      (inactive '()) . rest)
+			      (inactive '()) #&allow-other-keys #&rest rest)
+  "Set the border style in the current decor.
+This function takes the keyword arguments #:hidden-handles, 
+#:no-inset, and #:pixmap, with effects as described under the
+`Face Flags' and `Face Specification Flags' concepts (except
+that #:pixmap is always tiled).  These keyword arguments can
+either be included in the main argument list, or lists of these
+arguments can be used as the ACTIVE or INACTIVE arguments."
   (act-on-face-specs (lambda* (active #&optional ignore inactive)
 			      (if (bound? inactive)
 				  (set-border-face! active inactive)
@@ -49,11 +82,41 @@
 		     parse-border-face-specs parse-border-face-flags
 		     rest active '() inactive))
 
+(define*-public (set-left-button-face! button active-up #&optional
+				      (active-down #f) (inactive #f))
+  "Set the button face for the left-button numbered BUTTON.
+E.g., if BUTTON is 1, this will set the leftmost button of your
+titlebar.  See `set-button-face!' for a description of ACTIVE-UP,
+ACTIVE-DOWN, and INACTIVE."
+  (set-button-face! (+ (* (- button 1) 2) 1)
+		    active-up active-down inactive))
+
+(define*-public (set-right-button-face! button active-up #&optional
+				       (active-down #f) (inactive #f))
+  "Set the button face for the right-button numbered BUTTON.
+E.g., if BUTTON is 1, this will set the rightmost button of your
+titlebar.  See `set-button-face!' for a description of ACTIVE-UP,
+ACTIVE-DOWN, and INACTIVE."
+  (set-button-face! (+ (* (- button 1) 2) 2)
+		    active-up active-down inactive))
+
 
 (define*-public (button-style button #&key mwm
 			      (active-up '()) 
 			      (active-down '()) 
-			      (inactive '()) . rest)
+			      (inactive '()) #&allow-other-keys #&rest rest)
+  "Set the button style for button number BUTTON in the current decor.
+MWM sets the button's mwm flag (see `set-button-mwm-flag!'.
+This function also takes the keyword arguments #:justify, #:relief,
+#:vertical-justify, #:use-style-of, #:solid, #:gradient, #:h-gradient,
+#:v-gradient, #:relief-pattern, #:vector, #:pixmap, and #:tiled-pixmap,
+with effects as described under the `Face Flags' and `Face
+Specification Flags' concepts (with the additions that #:tiled-pixmap
+is an always-tiled pixmap, #:vector is a synonym for #:relief-pattern,
+and #:h-gradient and #:v-gradient are horizontal and vertical
+gradients).  These extra keyword arguments can either be included
+in the main argument list, or lists of these arguments can be used
+as the ACTIVE-UP, ACTIVE-DOWN, or INACTIVE arguments."
   (if (bound? mwm)
       (set-button-mwm-flag! mwm))
   (act-on-face-specs (lambda args
@@ -159,5 +222,3 @@
 	    (setter-proc (make-face active-up-flags active-up-specs)
 			 (make-face active-down-flags active-down-specs)
 			 (make-face inactive-flags inactive-specs))))))
-
-
