@@ -1,4 +1,5 @@
-;;;; 	Copyright (C) 1997 Maciej Stachowiak
+;;;; $Id: fvwm-compat.scm,v 1.13 2000/01/13 07:28:07 gjb Exp $
+;;;; Copyright (C) 1997, 1998, 1999, 2000 Maciej Stachowiak and Greg J. Badros
 ;;;; 
 ;;;; This program is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -19,6 +20,7 @@
 
 
 (define-module (app scwm fvwm-compat)
+  :use-module (app scwm base)
   :use-module (app scwm optargs)
   :use-module (app scwm winlist))
 
@@ -28,6 +30,8 @@
 (define fvwm-exec-shell "/bin/sh")
 
 (define*-public (fvwm-exec-use-shell #&optional (shell #f))
+  "Use SHELL when emulating fvwm \"EXEC\" commands.
+Defaults to <envar>$SHELL</envar> or <filename>/bin/sh</filename>."
   (set! fvwm-exec-shell 
 	(cond
 	 (command => id)
@@ -37,6 +41,8 @@
 
 
 (define-public (fvwm-exec command)
+  "Run COMMAND as fvwm would.
+See also `fvwm-exec-use-shell'."
   (if (eq? 0 (primitive-fork))
       (catch #t (lambda ()
 		  (execl
@@ -44,10 +50,6 @@
 		   (string-append "exec " command)))
 	     (lambda args (primitive-exit 100)))))
 
-(define*-public (fvwm-none thunk #&key (only '()) (except '()))
-  (if (null? (list-windows #:only only #:except except))
-      (thunk)))
-
 (define-public (fvwm-pipe-read command)
   (let* ((command-pipes (pipe))
 	 (read-pipe (car command-pipes))
@@ -77,7 +79,8 @@
 ;; guile probably permits us access to a lot of these
 ;; things, but some new primitives may need to be added
 ;; --03/10/98 gjb
-(define-public TWM_TYPE "fvwm")  ;; FIXGJB should this be scwm?
+(define-public TWM_TYPE "scwm")
+;;; GJB:FIXME:: new primitives or use guile to implement the below
 ;(define-public SERVERHOST (X-server-host-name))
 ;(define-public CLIENTHOST (X-client-host-name))
 ;(define-public HOSTNAME (process-host-name))
@@ -91,8 +94,8 @@
 (define-public REVISION (cadr X-version-info))
 (define-public VENDOR (caddr X-version-info))
 (define-public RELEASE (cadddr X-version-info))
-(define-public WIDTH (car (display-size)))
-(define-public HEIGHT (cadr (display-size)))
+(define-public WIDTH display-width)
+(define-public HEIGHT display-height)
 
 ;;(define (resolution pixels mm) 
 ;;  (/ (+ (/ (* pixels 100000) mm) 50) 100))
@@ -103,7 +106,7 @@
 
 (define-public X_RESOLUTION (list-ref X-display-info 0))
 (define-public Y_RESOLUTION (list-ref X-display-info 1))
-(define-public PLANES (list-ref X-display-info 2))
+(define-public PLANES (list-ref X-display-info 2))  ;; also `display-depth'
 (define-public BITS_PER_RGB (list-ref X-display-info 3))
 (define-public CLASS (list-ref X-display-info 4))
 (define-public COLOR (if (list-ref X-display-info 5) "Yes" "No"))
@@ -111,3 +114,16 @@
 (define-public FVWM_VERSION "3") ;; for lack of a better number
 (define-public OPTIONS "SHAPE XPM")
 ;(define-public FVWMDIR (default-directory))
+
+
+;; Perhaps this should be the default behaviour of %x?
+(define-public (%x-permit-negative x)
+  (if (>= x 0)
+      (%x x)
+      (x- (%x (- x)))))
+
+;; Perhaps this should be the default behaviour of %y?
+(define-public (%y-permit-negative y)
+  (if (>= y 0)
+      (%y y)
+      (y- (%y (- y)))))
