@@ -1,4 +1,5 @@
-;;;; 	Copyright (C) 1998 Maciej Stachowiak
+;;;; $Id: fvwm-module.scm,v 1.50 2000/01/13 07:28:07 gjb Exp $
+;;;; Copyright (C) 1998, 1999, 2000 Maciej Stachowiak and Greg J. Badros
 ;;;; 
 ;;;; This program is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -20,16 +21,22 @@
 
 (define-module (app scwm fvwm-module)
   :use-module (app scwm winlist)
+  :use-module (app scwm wininfo)
   :use-module (app scwm base)
+  :use-module (app scwm file)
+  :use-module (app scwm defoption)
   :use-module (app scwm module-types)
   :use-module (app scwm bincomm)
   :use-module (app scwm fvwm-eval)
   :use-module (app scwm optargs)
   :use-module (app scwm decor)
-  :use-module (ice-9 common-list))
+  :use-module (app scwm file)
+  :use-module (app scwm listops))
 
 
 
+(define-scwm-group fvwm2-module "Fvwm2 Modules")
+
 ;;   fvwm-module
 ;; Supports running fvwm2 modules from scwm - this is mostly useful for
 ;; the pager.
@@ -88,7 +95,7 @@
 ;; ;; Make sure the right place is in the module path. Note: the
 ;; ;; directory listed here is for example purposes only, it is actually
 ;; ;; in the default module path.
-;; (set! fvwm2-module-path (append fvwm2-module-path '("/usr/lib/X11/fvwm2")))
+;; (set! *fvwm2-module-path* (append *fvwm2-module-path* '("/usr/lib/X11/fvwm2")))
 ;; ;; Register the module configuration for the pager.
 ;; (register-fvwm2-module-config "FvwmPager"
 ;;  			         "*FvwmPagerBack grey76"
@@ -112,21 +119,31 @@
 ;;   (kill-fvwm2-modules-by-name "FvwmPager")
 ;;
 ;; This support is in progress. It may or may not work with particular
-;; modules, depending on wether all the commands they use are
+;; modules, depending on whether all the commands they use are
 ;; supported by the fvwm-eval module (q.v.).
 ;;
 ;;
 
 
+(define-scwm-option *debug-fvwm-module* #f
+  "Set this if you want debugging output from fvwm2 modules."
+  #:type 'boolean
+  #:group 'fvwm2-module)
+
+
+(define-scwm-option *fvwm2-module-path* '("/usr/lib/X11/fvwm2"
+					"/usr/local/lib/X11/fvwm2")
+  "Set this to the path where your fvwm2 module binaries live."
+  #:type 'path
+  #:group 'fvwm2-module)
+
+
 ;; FIXMS: Maybe scwm should always ignore sigpipes?
 (sigaction SIGPIPE SIG_IGN)
 
 (define app-window "0")
 (define context "0") ; C_NO_CONTEXT
 
-(define display-width (car (display-size)))
-(define display-height (cadr (display-size)))
-
 (define (fvwm2-module-read-packet port)
   (define (wait-for-input)
     (if (char-ready? port)
@@ -139,17 +156,37 @@
   (wait-for-input)
   (let* ((window
 	  (catch #t (lambda () (binary-read-long port))
-		 (lambda args (error "Could not get window id"))))
+		 (lambda args (error 
+                               (string-append "Could not get window id:\n"
+                                              (call-with-output-string
+                                               (lambda ()
+                                                 (write args)
+                                                 (newline))))))))
 	 (msglen
 	  (catch #t (lambda () (binary-read-long port))
-		 (lambda args (error "Could not get msglen"))))
+		 (lambda args (error 
+                               (string-append "Could not get msglen:\n"
+                                              (call-with-output-string
+                                               (lambda ()
+                                                 (write args)
+                                                 (newline))))))))
 	 (command
 	  (catch #t (lambda () (binary-read msglen port))
-		 (lambda args (error "Could not get command"))))
+		 (lambda args (error 
+                               (string-append "Could not get command:\n"
+                                              (call-with-output-string
+                                               (lambda ()
+                                                 (write args)
+                                                 (newline))))))))
 	 (keepgoing
 	  (catch #t (lambda () (binary-read-long port))
-		 (lambda args (error "Could not get keepgoing")))))
-	 (list window msglen command keepgoing)))
+		 (lambda args (error 
+                               (string-append "Could not keepgoing:\n"
+                                              (call-with-output-string
+                                               (lambda ()
+                                                 (write args)
+                                                 (newline)))))))))
+    (list window msglen command keepgoing)))
 
 (define (fvwm2-module-send-config-info config-info port)
   (map (lambda (x) (send-config-info x port))
@@ -167,7 +204,9 @@
   (if (not (= (modulo (string-length body)  4) 0))
       (error "Bad packet length"))
   (binary-write 
-   (string-append (make-packet-header id (/ (string-length body) 4))
+   (string-append (make-packet-header 
+		   id 
+		   (inexact->exact (round (/ (string-length body) 4))))
 		  body) port)
   (force-output port))
 
@@ -214,7 +253,7 @@
 			(* (- (cadr (desk-size)) 1) display-height))))
    port)
 
-  (let ((focus-win (current-window-with-focus)))
+  (let ((focus-win (window-with-focus)))
     (fvwm2-module-send-packet
      M_FOCUS_CHANGE
      (longs->string
@@ -224,10 +263,10 @@
       ;; Fvwm always uses the default decor to send this; perhaps we should
       ;; use the window's decor instead?
       (with-decor (default-decor)
-		  (color-property (hilight-foreground) 
+		  (color-property (highlight-foreground) 
 				  'pixel))
       (with-decor (default-decor)		   
-		  (color-property (hilight-background) 
+		  (color-property (highlight-background) 
 				  'pixel)))
      port))
   
@@ -261,9 +300,9 @@
     (send-win-string M_RES_CLASS (window-class win))
     (send-win-string M_RES_NAME (window-resource win))
   
-    (if (iconified? win)
+    (if (iconified-window? win)
     	(fvwm2-module-send-packet 
-    	 M_ICONIFY_WINDOW 
+    	 M_ICONIFY 
     	 (marshal-fvwm2-iconify-info win)
     	 port))
 
@@ -281,17 +320,40 @@
 (define (remove-active-module! fmod)
   (set! active-modules (delq! fmod active-modules)))
 
-(define (module-broadcast type num-data . args)
+;; CRW:FIXME:MS: The (logior type mask) below doesn't make any sense.
+;; 1) Shouldn't it be some sort of "and" instead of "or"?
+;; 2) logior will never return #f, so the "true" branch will always be taken...
+;; (There are several other functions in this file with the same problem.)
+
+;; Alexander Vorobiev: It seems (not (zero? (logand type mask))) should
+;; be used. I corrected all relevant functions.
+
+(define (module-broadcast type num-data arg1 arg2 arg3 arg4 arg5 arg6 arg7)
+  (if (optget *debug-fvwm-module*)
+      (begin
+	   (display "broadcasting message of type: ")
+	   (write type)
+	   (newline)))
+  
   (map (lambda (fmod)
 	 (let ((to-module-write (car fmod))
-	       (mask (cadr fmod)))
-	   (if (logior type mask)
+	       (mask (cadr fmod))
+	       (args (list arg1 arg2 arg3 arg4 arg5 arg6 arg7)))
+	   (set-cdr! (list-tail args (- num-data 1)) '())
+	   (if (optget *debug-fvwm-module*)
+   	       (begin
+	          (display "mask: ")
+	          (write mask)
+	          (newline)
+		  (display "(logand type mask): ")
+		  (write (logand type mask))
+		  (newline)))
+
+	   (if (not (zero? (logand type mask)))
 	       (fvwm2-module-send-packet 
 		type
 		(apply string-append
-		       (map (lambda (x y) 
-			      (long->string x)) 
-			    args (iota num-data))) 
+		       (map long->string args))
 		to-module-write))))
        active-modules))
 
@@ -301,7 +363,7 @@
   (map (lambda (fmod)
 	 (let ((to-module-write (car fmod))
 	       (mask (cadr fmod)))
-	   (if (logior type mask)
+	   (if (not (zero? (logand type mask)))
 	       (fvwm2-module-send-packet 
 		type
 		(marshal-fvwm2-config-info window)
@@ -314,7 +376,7 @@
   (map (lambda (fmod)
 	 (let ((to-module-write (car fmod))
 	       (mask (cadr fmod)))
-	   (if (logior type mask)
+	   (if (not (zero? (logand type mask)))
 	       (fvwm2-module-send-packet 
 		type
 		(pad-string-to-long
@@ -332,37 +394,12 @@
   (map (lambda (fmod)
 	 (let ((to-module-write (car fmod))
 	       (mask (cadr fmod)))
-	   (if (logior type mask)
+	   (if (not (zero? (logand type mask)))
 	       (send-mini-icon-packet window to-module-write))))
        active-modules))
 
 (add-hook! broadcast-mini-icon-hook module-broadcast-mini-icon)
 
-;;; FIXMS: The next two should be more generally available!
-(define (filename-is-full-path? fname)
-  (and (> (string-length fname) 0)
-       (or (char=? (string-ref fname 0) #\/)
-	   (and (char=? (string-ref fname 0) #\.)
-		(or
-		 (and (> (string-length fname 1))
-		      (char=? (string-ref fname 1) #\/))
-		 (and (> (string-length fname 2))
-		      (char=? (string-ref fname 1) #\.)
-		      (char=? (string-ref fname 2) #\/)))))))
-			 
-(define (find-file-in-path fname path)
-  (if (filename-is-full-path? fname)
-      (if (and (file-exists? fname)
-	       (not (file-is-directory? fname)))
-	  fname
-	  #f)
-      (or-map (lambda (prefix)
-		(let ((fp (string-append prefix "/" fname)))
-		  (if (and (file-exists? fp)
-			   (not (file-is-directory? fp)))
-		      fp
-		      #f))) 
-	      path)))
 
 (define fvwm2-module-config-hash (make-hash-table 5))
 
@@ -383,19 +420,23 @@
 (define-public (clear-fvwm2-module-config module-type)
   (hash-remove! fvwm2-module-config-hash module-type))
 
-
-(define-public fvwm2-module-path '("/usr/lib/X11/fvwm2"
-				   "/usr/local/lib/X11/fvwm2"))
-
+(define (aux-config-info)
+  (let ((path (path-list->string-with-colons image-load-path)))
+    (list
+     (string-append "IconPath " path)
+     (string-append "PixmapPath " path)
+     "ColorLimit 0"
+     "ClickTime 150")))
 
 (define*-public (run-fvwm2-module module-name #&optional
 				  (other-args '())	       
 				  (config-file "~/.fvwm2rc")
 				  (config-info
-				   (get-fvwm2-module-config
-				    (basename module-name))))
+				   (append (aux-config-info)
+                                           (get-fvwm2-module-config
+					    (basename module-name)))))
   (let ((module-file (find-file-in-path module-name
-					fvwm2-module-path)))
+					*fvwm2-module-path*)))
     (if module-file
 	(let* ((from-module-pipe (pipe))
 	       (from-module-read (car from-module-pipe))
@@ -404,7 +445,7 @@
 	       (to-module-read (car to-module-pipe))
 	       (to-module-write (cdr to-module-pipe))
 	       (input-hook-handle #f)
-	       ;; FIXMS: should use a struct, not a list.
+	       ;; MS:FIXME:: should use a struct, not a list.
 	       (fmod (list to-module-write 0
 			   (lambda ()
 			     (fvwm2-module-send-config-info 
@@ -466,9 +507,12 @@
 	  (fcntl to-module-write F_SETFD 1)
 	  (fcntl from-module-read F_SETFD 1)
 	  
-	  ;; FIXGJB: set o_nonblock for TO_MODULE_WR
-	  ;; GJBFIX: why is this necessary?
-	  ;; (fctl to-module-write F_SETFL O_NONBLOCK)
+	  ;; set o_nonblock for TO_MODULE_WR
+	  ;; w/o this scwm will block when writing to a module
+	  ;; pipe if that module is not reading;  observed
+	  ;; effect to scwm is a hang that can be eliminated
+	  ;; by killing the modules
+	  (fcntl to-module-write F_SETFL O_NONBLOCK)
 	  
 	  (add-active-module! fmod)
 	  (list-set! fmod 4 #t)
@@ -482,10 +526,20 @@
 					   from-module-read))
 				  (window-id (car packet))
 				  (command (caddr packet)))	
-			     
-			     ;; (display "packet: ")
-			     ;; (write packet)
-			     ;; (newline)
+			     (if (optget *debug-fvwm-module*)
+				 (begin
+				   (display "packet: ")
+				   (write packet)
+				   (newline)
+				   (display "command =")
+				   (write command)
+				   (newline)
+				   (display "fmod = ")
+				   (write fmod)
+				   (newline)
+				   (display "window-id = ")
+				   (write window-id)
+				   (newline)))
 			     (catch #t
 				    (lambda()
 				      (if (= window-id 0)
@@ -494,6 +548,7 @@
 							     fmod
 							     (id->window 
 							      window-id))))
+				    ;; GJB:FIXME:MS: Can we get a better error?
 				    (lambda args (display 
 						  "Error evaling packet: ")
 					    (write packet)
@@ -502,9 +557,14 @@
 			     (if (not (list-ref fmod 4))
 				 (remove-input-hook! input-hook-handle))))
 			 (lambda args
-			   (display "scwm: Error communicating with module ")
+			   (display "scwm: Error communicating with module: \n")
 			   (write fmod)
-			   (display ";\n terminating connection.\n")
+			   (newline)
+                           (display " error is: \n")
+                           (write args)
+			   (newline)
+			   (backtrace)
+			   (display " terminating connection.\n")
 			   (kill-fvwm2-module fmod))))))
 	    
 	    (set! input-hook-handle (add-input-hook! from-module-read 
@@ -517,11 +577,36 @@
   ((list-ref fmod 5)))
        
 (define-public (kill-all-fvwm2-modules)
-  (map kill-fvwm2-module active-modules))
+  (for-each kill-fvwm2-module active-modules))
 
 (define-public (kill-fvwm2-modules-by-name module-name)
-  (map kill-fvwm2-module (pick (lambda (fmod)  
-				(string=? (list-ref fmod 6) module-name)) 
-				active-modules)))
+  (for-each kill-fvwm2-module (filter (lambda (fmod)  
+					(string=? (list-ref fmod 6) module-name)) 
+				      active-modules)))
 
-(add-hook! shutdown-hook kill-all-fvwm2-modules)
+(add-hook! shutdown-hook
+	   (lambda (restarting?)
+	     (kill-all-fvwm2-modules)))
+
+
+(define-public (fvwm2-pager-window)
+  "Return a fvwm2 pager window, or #f if there is none."
+  (let ((pagers
+	 (list-windows #:only (win-and?? (class-match?? "FvwmModule") 
+					 (resource-match?? "FvwmPager")))))
+    (and (pair? pagers) (car pagers))))
+
+(define*-public (raise-fvwm2-pager)
+  "Raise a fvwm2 pager window in the stacking order."
+  (interactive)
+  (let ((w (fvwm2-pager-window)))
+    (and w (raise-window w))))
+
+(define*-public (lower-fvwm2-pager)
+  "Lower a fvwm2 pager window in the stacking order."
+  (interactive)
+  (let ((w (fvwm2-pager-window)))
+    (and w (lower-window w))))
+
+;; (deiconify-window (fvwm2-pager-window))
+;; (iconify-window (fvwm2-pager-window))
